"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
var ts_morph_1 = require("ts-morph");
var default_1 = require("./default");
var util_1 = require("./util");
var TAG_NAMES_FOR_DESCRIPTION = ['zh', 'en'];
var project = new ts_morph_1.Project({
    compilerOptions: {
        jsx: 'react',
    },
});
var propertyRegex = /(\w+)\s{0,}([?]?)\s{0,}:(.*?);?$/s;
// extract pure type
function extractFromPropertyText(text) {
    var regexResult = propertyRegex.exec(text);
    if (!regexResult) {
        return;
    }
    var name = regexResult[1];
    var isOptional = regexResult[2] === '?';
    var type = (0, util_1.toSingleLine)(regexResult[3]);
    return {
        name: name,
        isOptional: isOptional,
        type: type,
    };
}
// Get key-value pairs from jsDoc of Declarations
function getDeclarationTags(declaration) {
    var _a;
    var tags = [];
    var rawTags = ((_a = declaration.getJsDocs()[0]) === null || _a === void 0 ? void 0 : _a.getTags()) || [];
    var title;
    for (var _i = 0, rawTags_1 = rawTags; _i < rawTags_1.length; _i++) {
        var tag = rawTags_1[_i];
        var name = tag.getTagName();
        var value = tag.getCommentText() || '';
        if (name === 'title') {
            title = value;
        }
        tags.push({ name: name, value: value });
    }
    return {
        title: title,
        tags: tags,
    };
}
// Get key-value pairs from jsDoc of Symbol
function getSymbolTags(sym, strictComment) {
    if (strictComment === void 0) { strictComment = false; }
    var jsDocTags = sym.compilerSymbol.getJsDocTags();
    var tags = jsDocTags.map(function (tag) {
        var _a;
        return ({
            name: tag.name,
            value: ((_a = tag.text) === null || _a === void 0 ? void 0 : _a[0].text) || '',
        });
    });
    // Try to extend property description from common comment
    if (!strictComment) {
        var commonComment_1 = sym.compilerSymbol.getDocumentationComment(undefined)[0];
        if (commonComment_1 && commonComment_1.kind === 'text' && commonComment_1.text) {
            TAG_NAMES_FOR_DESCRIPTION.forEach(function (tagNameForDescription) {
                if (!tags.find(function (_a) {
                    var name = _a.name;
                    return name === tagNameForDescription;
                })) {
                    tags.push({ name: tagNameForDescription, value: commonComment_1.text });
                }
            });
        }
    }
    return tags;
}
// Get Json schema of interface's property
function getPropertySchema(sym, defaultT, strictComment) {
    if (strictComment === void 0) { strictComment = false; }
    var name = sym.getName();
    var typeText = sym.getDeclarations()[0].getText();
    var extract = extractFromPropertyText(typeText);
    if (!extract) {
        return null;
    }
    var tags = getSymbolTags(sym, strictComment);
    if (tags.find(function (_a) {
        var name = _a.name;
        return name && TAG_NAMES_FOR_DESCRIPTION.indexOf(name) > -1;
    })) {
        return {
            name: name,
            type: extract.type,
            isOptional: extract.isOptional,
            tags: tags,
        };
    }
    return defaultT[name]
        ? __assign({ name: name, isOptional: extract.isOptional }, defaultT[name]) : null;
}
// Get Json schema of Function
function getFunctionSchema(declaration, strictComment) {
    if (strictComment === void 0) { strictComment = false; }
    return {
        params: declaration.getParameters().map(function (para) {
            var _a, _b;
            var tags = getSymbolTags(para.getSymbol(), strictComment);
            return {
                tags: tags,
                name: para.getName(),
                type: para.getType().getText(),
                isOptional: para.isOptional(),
                initializerText: ((_a = para.getInitializer()) === null || _a === void 0 ? void 0 : _a.getText()) ||
                    ((_b = tags.find(function (_a) {
                        var name = _a.name;
                        return name === 'default' || name === 'defaultValue';
                    })) === null || _b === void 0 ? void 0 : _b.value) ||
                    null,
            };
        }),
        returns: declaration.getReturnType().getText(),
    };
}
function generateSchema(sourceFile, typeChecker, config) {
    var interfaces = (sourceFile === null || sourceFile === void 0 ? void 0 : sourceFile.getInterfaces()) || [];
    var typeAliases = (sourceFile === null || sourceFile === void 0 ? void 0 : sourceFile.getTypeAliases()) || [];
    var functions = (sourceFile === null || sourceFile === void 0 ? void 0 : sourceFile.getFunctions()) || [];
    var defaultT = (config === null || config === void 0 ? void 0 : config.defaultTypeMap) || default_1.defaultTypeMap;
    var strictComment = !!(config === null || config === void 0 ? void 0 : config.strictComment);
    var propertySorter = config === null || config === void 0 ? void 0 : config.propertySorter;
    var schemaMap = {};
    var schemaList = [];
    __spreadArray(__spreadArray(__spreadArray([], interfaces, true), typeAliases, true), functions, true).sort(function (declarationA, declarationB) {
        return declarationA.getStartLineNumber() - declarationB.getStartLineNumber();
    })
        .forEach(function (declaration) {
        var _a, _b;
        var _c = getDeclarationTags(declaration), title = _c.title, tags = _c.tags;
        var dType = declaration.getKindName();
        if (!title) {
            return;
        }
        var schema;
        var typeNode = dType === 'FunctionDeclaration'
            ? declaration
            : dType === 'TypeAliasDeclaration'
                ? declaration.getTypeNode()
                : null;
        // Function declaration
        if (typeNode &&
            ['FunctionDeclaration', 'FunctionType'].indexOf(typeNode.getKindName()) > -1) {
            schema = __assign({ tags: tags }, getFunctionSchema(typeNode));
        }
        // Interface declaration forbid extends
        else if (dType === 'InterfaceDeclaration' &&
            !!tags.find(function (_a) {
                var name = _a.name;
                return name === 'notExtends';
            })) {
            var data_1 = [];
            declaration.getProperties().forEach(function (a) {
                var schema = getPropertySchema(a.getSymbol(), defaultT, strictComment);
                schema && data_1.push(schema);
            });
            schema = { tags: tags, data: data_1 };
        }
        else {
            var data_2 = [];
            typeChecker.getPropertiesOfType(declaration.getType()).forEach(function (a) {
                var schema = getPropertySchema(a, defaultT, strictComment);
                schema && data_2.push(schema);
            });
            schema = { tags: tags, data: data_2 };
        }
        if (typeof propertySorter === 'function') {
            (_a = schema.data) === null || _a === void 0 ? void 0 : _a.sort(propertySorter);
            (_b = schema.params) === null || _b === void 0 ? void 0 : _b.sort(propertySorter);
        }
        schemaList.push({ title: title, schema: schema });
        schemaMap[title] = schema;
    });
    return (config === null || config === void 0 ? void 0 : config.strictDeclarationOrder) ? schemaList : schemaMap;
}
function generate(file, config) {
    if (config === null || config === void 0 ? void 0 : config.sourceFilesPaths) {
        project.addSourceFilesAtPaths(config === null || config === void 0 ? void 0 : config.sourceFilesPaths);
    }
    var typeChecker = project.getTypeChecker();
    var sourceFile = project.getSourceFile(file);
    if (!sourceFile) {
        return;
    }
    return generateSchema(sourceFile, typeChecker, config);
}
exports.default = generate;
